{ config, lib, modulesPath, pkgs, ... }:

{
  boot.kernelParams = [ "boot.shell_on_fail" ];
  home-manager.users.${config.lwad.definitions.username} = {
    home.stateVersion = "23.05";
    programs.librewolf.settings."network.trr.mode" = lib.mkForce 0;
  };
  imports = [
    "${modulesPath}/virtualisation/qemu-vm.nix"
    "${modulesPath}/profiles/qemu-guest.nix"
    ../components/desktop-environment.nix
    ../lwad
  ];
  lwad = {
    bootLoader.enable = false;
    definitions.sudoas.variant = "sudo";
  };
  networking.hostName = "asuna";
  programs = {
    bash.shellAliases = let
      gitAuth = pkgs.writeScript "gitAuth" ''
        #! ${pkgs.expect}/bin/expect -f

        set pass [exec cat /home/${config.lwad.definitions.username}/Documents/gitkey]
        set env(GIT_ASKPASS) ""

        set force_conservative 0

        if {$force_conservative} {
          set send_slow {1 .1}
          proc send {ignore arg} {
            sleep .1
            exp_send -s -- $arg
          }
        }

        set timeout -1
        spawn git {*}$argv
        match_max 100000
        expect -exact "Password for 'https://lwad@git.disroot.org': "
        send -- "$pass\r"
        expect eof
      '';
      gitFetch = pkgs.writeScript "gitFetch" ''
        ${gitAuth} clone "https://lwad@git.disroot.org/lwad/$1"
      '';
    in {
      "getch" = "${gitFetch}";
      "glone" = "${gitAuth} clone";
      "gull" = "${gitAuth} pull";
      "gush" = "${gitAuth} push";
      "ts" =
        "${config.lwad.definitions.sudoas.command} tailscale up --auth-key 'file:/home/${config.lwad.definitions.username}/Documents/tskey'";
    };
    tmux.extraConfigBeforePlugins = ''
      set-option -g status-right ""
    '';
  };
  services = {
    logrotate.enable = false;
    unclutter-xfixes.enable = lib.mkForce false;
  };
  system.stateVersion = "23.05";
  systemd.tmpfiles.rules = [
    "Z /home/${config.lwad.definitions.username}/Documents - ${config.lwad.definitions.username} users"
    "L+ /home/${config.lwad.definitions.username}/nixos - - - - /home/${config.lwad.definitions.username}/Documents/asuna-vm"
  ];
  users.users.${config.lwad.definitions.username}.hashedPassword =
    "$y$j9T$2WsVbq7A5SXrOrdJHKG4y1$NHnBWQ1jeiGyaxgNysGyNPhLJjW1BQlAWpNx4TdQ2L8";
  virtualisation = {
    diskImage = "$HOME/asuna-vm.qcow2";
    diskSize = (64 * 1024);
    fileSystems."/home/${config.lwad.definitions.username}/Documents" = {
      device = "h-drive";
      fsType = "9p";
      options = [ "trans=virtio" "version=9p2000.L" "access=user" "nofail" ];
    };
    qemu.options = [
      "-vga virtio"
      ''-smp "$(nproc)"''
      ''
        -m "$(($(getconf _PHYS_PAGES) * $(getconf PAGE_SIZE) / (1024 ** 2) * 3 / 4))"''
      "-full-screen"
      ''
        $(if [ -d "$HOME/h-drive" ]; then OPTION="-virtfs local,path=$HOME/h-drive,security_model=mapped-file,mount_tag=h-drive"; else OPTION=""; fi; echo "$OPTION")''
      "-display gtk,grab-on-hover=yes,full-screen=yes"
    ];
    resolution = {
      x = 2560;
      y = 1080;
    };
  };
}
