{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.git.enable = mkEnableOption "git";
  config = mkIf config.lwad.applications.git.enable {
    home-manager.users.${config.lwad.definitions.username}.programs.git = {
      enable = true;
      userEmail = "toolbox.0-kindred@icloud.com";
      userName = "lwad";
    };
    programs = {
      bash.interactiveShellInit = let git = "${pkgs.git}/bin/git";
      in ''
        git() {
          if [[ "$1" == "commit" ]] && [ -f "$(${git} rev-parse --show-toplevel)/.precommit" ]; then
            $(${git} rev-parse --show-toplevel)/.precommit
          fi
          ${git} "$@"
        }

        g() {
          ${
            if config.lwad.applications.neovim.enable then ''
              if [ -z "$1" ]
              then
                nvim -c "G | bd 1"
              else
                git "$@"
              fi
            '' else
              ''git "$@"''
          }
        }

        source ${pkgs.git}/share/bash-completion/completions/git
        complete -o bashdefault -o default -o nospace -F __git_wrap__git_main g
      '';
      git = {
        config = {
          core.excludesFile = pkgs.writeText "gitignore" ''
            .virtfs_metadata
            .run
          '';
          init.defaultBranch = "main";
          push.autoSetupRemote = true;
        };
        enable = true;
      };
    };
  };
}
