{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.i3.displayProperties = mkOption {
    default = { };
    description = "Displays to use in the display menu.";
    type = types.attrsOf (types.submodule {
      options = {
        extraArgs = mkOption {
          default = null;
          description = "Extra arguments to pass to `xrandr`.";
          type = types.nullOr types.str;
        };
        height = mkOption {
          description = "Height of the display.";
          type = types.ints.unsigned;
        };
        output = mkOption {
          description = "Output the display is connected to.";
          type = types.str;
        };
        width = mkOption {
          description = "Width of the display.";
          type = types.ints.unsigned;
        };
      };
    });
  };
  config.home-manager.users.${config.lwad.definitions.username}.xsession.windowManager.i3.config.keybindings =
    let
      cfg = config.lwad.applications.i3;
      mod = cfg.modifiers.primary;
    in {
      "${mod}+o" = "exec --no-startup-id ${
          pkgs.writeScript "display_mode_changer.py" ''
            #! /usr/bin/env python

            import subprocess
            from functools import reduce as red

            DISPLAY_PROPERTIES = {
            ${concatMapStrings (val: "    " + val + "\n") (attrValues (mapAttrs
              (key: value:
                ''
                  "${key}": (${toString value.width}, ${
                    toString value.height
                  }, "${value.output}", "${
                    if (value.extraArgs != null) then
                      (replaceStrings [ ''"'' ] [ ''\"'' ] value.extraArgs)
                    else
                      ""
                  }"),'') cfg.displayProperties))}
            }
            displays = list(DISPLAY_PROPERTIES.keys())

            hdmiOutputName = ""


            def getDisplay(displays, prompt):
                rofiInput = "\n".join(displays)
                return subprocess.run(input=rofiInput,
                                      executable="/bin/sh",
                                      args=f"rofi -dmenu -p \"{prompt}\" -i",
                                      shell=True,
                                      capture_output=True,
                                      text=True).stdout.strip()


            def getPart(display, extraArgs=""):
                if display == "":
                    return ""

                if display == "internal":
                    position = "0x0"
                else:
                    position = f"{DISPLAY_PROPERTIES['internal'][0]}x0"

                outputName = subprocess.run(
                    executable="/bin/sh",
                    args=
                    f"xrandr | rg \"{DISPLAY_PROPERTIES[display][2]}\" | cut -d \" \" -f 1",
                    shell=True,
                    capture_output=True,
                    text=True).stdout.strip()

                properties = DISPLAY_PROPERTIES[display]
                return f" --output {outputName} --mode {properties[0]}x{properties[1]} --pos {position} {properties[3]} {extraArgs}"


            if __name__ == "__main__":
                primary = getDisplay(displays, "Primary Display")
                if primary == "":
                    exit(1)

                displays.remove(primary)
                secondary = getDisplay(displays, "Secondary Display")

                activeMonitors = subprocess.run(
                    executable="/bin/sh",
                    args=
                    "xrandr --listactivemonitors | cut -d \" \" -f 3 | awk 'NF'",
                    shell=True,
                    capture_output=True,
                    text=True).stdout.strip().split("\n")

                command = "xrandr" + red(lambda acc, val: acc + f" --output {val.removeprefix('+').removeprefix('*')} --off",
                                         activeMonitors, "") + " && xrandr" + getPart(
                                             primary, "--primary") + getPart(secondary)
                subprocess.run(executable="/bin/sh",
                               args=command,
                               shell=True,
                               capture_output=True,
                               text=True)
          ''
        }";
    };
}
