{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.i3.moveDisplayUp.enable = mkEnableOption
    "a script to move the secondary display to above the primary display";
  config.home-manager.users.${config.lwad.definitions.username}.xsession.windowManager.i3.config.keybindings."${config.lwad.applications.i3.modifiers.primary}+Shift+u" =
    mkIf config.lwad.applications.i3.moveDisplayUp.enable
    "exec --no-startup-id ${
      let
        cut = "${pkgs.coreutils}/bin/cut";
        rg = "${pkgs.ripgrep}/bin/rg";
        tail = "${pkgs.coreutils}/bin/tail";
        xrandr = "${pkgs.xorg.xrandr}/bin/xrandr";
      in pkgs.writeScript "move_display_up.sh" ''
        output="$(${xrandr} --listactivemonitors | ${tail} -n +2 | ${rg} -e " [0-9]*: \+[a-zA-Z0-9]" | ${cut} -d " " -f 3,4)"
        outputName="$(${cut} -d " " -f 1 <<< "''${output:1}")"
        outputProperties="$(${cut} -d " " -f 2 <<< "$output")"

        height="$(${cut} -d "x" -f 2 <<< "$outputProperties" | ${cut} -d "/" -f 1)"

        ${xrandr} --output "$outputName" --pos "0x-$height"
      ''
    }";
}
