{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.imunes.enable = mkEnableOption "imunes";
  config = mkIf config.lwad.applications.imunes.enable {
    environment.systemPackages = [
      (pkgs.tcl.mkTclDerivation rec {
        pname = "imunes";
        version = "2.4.0";

        src = pkgs.fetchFromGitHub {
          owner = "imunes";
          repo = "imunes";
          rev = "v${version}";
          hash = "sha256-6reA4mh5Ub8phz6v4wrlcDeEg3Ycn96i95ZUROtnHhU=";
        };

        buildFlags = [ "all" ];

        makeFlags = [ "PREFIX=$(out)" ];

        dontBuild = true;

        postInstall = ''
          makeWrapper "${pkgs.tcl}/bin/tclsh8.6" "$out/bin/imunes" --add-flags "$out/lib/imunes/imunes.tcl" --prefix PATH : ${
            makeBinPath [ pkgs.docker ]
          }
        '';

        nativeBuildInputs = [ pkgs.makeWrapper ];

        buildInputs = [ pkgs.tcllib pkgs.tk ];
      })
    ];
    virtualisation.docker.enable = true;
  };
}
