{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.librewolf.enable = mkEnableOption "librewolf";
  config = mkIf config.lwad.applications.librewolf.enable {
    home-manager.users.${config.lwad.definitions.username} = { lib, ... }: {
      home.activation.librewolfToFirefox =
        lib.hm.dag.entryAfter [ "linkGeneration" ] ''
          link_file="${
            config.home-manager.users.${config.lwad.definitions.username}.home.homeDirectory
          }/.librewolf"
          if ! [ -L "$link_file" ] || ! [[ "$(readlink "$link_file")" == ".mozilla/firefox/" ]]
          then
            if [ -e "$link_file" ]
            then
              mv "$link_file" "''${link_file}.backup"
            fi
            ln -s .mozilla/firefox "$link_file"
          fi
        '';
      programs.firefox = {
        enable = true;
        package = pkgs.librewolf;
        profiles.default = {
          id = 0;
          isDefault = true;
          search = {
            default = "DuckDuckGo";
            engines = {
              "disroot" = {
                definedAliases = [ "d" ];
                iconUpdateURL =
                  "https://git.disroot.org/assets/img/favicon.svg";
                urls = [{
                  template =
                    "https://git.disroot.org/lwad?sort=recentupdate&language=&q={searchTerms}&tab=repositories";
                }];
              };
              "mynixos" = {
                definedAliases = [ "m" ];
                iconUpdateURL = "https://mynixos.com/favicon-dark.svg";
                urls = [{
                  template = "https://mynixos.com/search?q={searchTerms}";
                }];
              };
              "nixpkgs" = {
                definedAliases = [ "n" ];
                icon =
                  "${pkgs.nixos-icons}/share/icons/hicolor/scalable/apps/nix-snowflake.svg";
                urls = [{
                  template = "https://search.nixos.org/packages";
                  params = [
                    {
                      name = "type";
                      value = "packages";
                    }
                    {
                      name = "query";
                      value = "{searchTerms}";
                    }
                  ];
                }];
              };
            };
            force = true;
          };
          settings = { # Convenience.
            "browser.startup.page" = 3;
            "identity.fxaccounts.enabled" = true;
            "toolkit.legacyUserProfileCustomizations.stylesheets" = true;
          } // { # Hardening.
            "accessibility.force_disabled" = 1;
            "app.shield.optoutstudies.enabled" = false;
            "beacon.enabled" = false;
            "browser.contentblocking.category" = "strict";
            "browser.newtab.preload" = false;
            "browser.newtabpage.enabled" = false;
            "browser.newtabpage.activity-stream.feeds.discoverystreamfeed" =
              false;
            "browser.newtabpage.activity-stream.section.highlights.includePocket" =
              false;
            "browser.places.speculativeConnect.enabled" = false;
            "browser.region.network.url" = "";
            "browser.region.update.enabled" = false;
            "browser.safebrowsing.allowOverride" = false;
            "browser.safebrowsing.downloads.remote.block_potentially_unwanted" =
              false;
            "browser.safebrowsing.downloads.remote.block_uncommon" = false;
            "browser.safebrowsing.downloads.remote.enabled" = false;
            "browser.safebrowsing.downloads.remote.url" = "";
            "browser.safebrowsing.provider.google4.dataSharingURL" = "";
            "captivedetect.canonicalURL" = "";
            "intl.accept_languages" = "en-US, en";
            "javascript.use_us_english_locale" = true;
            "media.autoplay.default" = 1;
            "media.eme.enabled" = false;
            "media.peerconnection.enabled" = false;
            "media.peerconnection.ice.default_address_only" = true;
            "media.peerconnection.ice.no_host" = true;
            "media.peerconnection.ice.proxy_only_if_behind_proxy" = true;
            "middlemouse.paste" = false;
            "network.captive-portal-service.enabled" = false;
            "network.dns.disablePrefetch" = true;
            "network.file.disable_unc_paths" = true;
            "network.gio.supported-protocols" = "";
            "network.http.referer.XOriginPolicy" = 2;
            "network.http.referer.XOriginTrimmingPolicy" = 2;
            "network.http.speculative-parallel-limit" = 0;
            "network.IDN_show_punycode" = true;
            "network.predictor.enabled" = false;
            "network.prefetch-next" = false;
            "network.trr.mode" = 3;
            "network.trr.custom_uri" = "1.1.1.1";
            "permissions.manager.defaultsUrl" = "";
            "webgl.disabled" = true;
          };
          userChrome = ''
            /* make tab bar smaller */
            :root {
              --tab-min-height: 0rem !important;
              --tab-block-margin: 0rem !important;
              --tab-border-radius: 0rem !important;
            }
            .tab-label-container {
              height: 1.8rem !important;
            }
            .tab-label-container > .tab-secondary-label {
              display: none !important;
            }

            /* remove spacing around tabs */
            .tabbrowser-tab {
              padding: 0rem !important;
              overflow-clip-margin: 0rem !important;
            }
            #tabbrowser-tabs[haspinnedtabs]:not([positionpinnedtabs])
              > #tabbrowser-arrowscrollbox
              > .tabbrowser-tab:nth-child(1 of :not([pinned], [hidden])) {
              margin-inline-start: 0rem !important;
            }

            /* make nav bar smaller */
            #nav-bar {
              font-size: 1rem !important;
              height: 1.8rem !important;
            }

            /* make bookmarks bar smaller */
            #PersonalToolbar {
              font-size: 1rem !important;
              height: 1.8rem !important;
            }

            /* remove useless components */
            #alltabs-button,
            #forward-button,
            #identity-box,
            #page-action-buttons > :not(#urlbar-zoom-button),
            .tab-label-container > .tab-secondary-label,
            #urlbar-go-button {
              display: none !important;
            }

            /* reduce spacing */
            .titlebar-buttonbox-container,
            .titlebar-spacer {
              display: none !important;
            }

            /* remove drop shadows */
            * {
              box-shadow: none !important;
            }

            /* stop overlapping in url bar */
            #urlbar-container {
              min-width: initial !important;
            }

            /* unify colors */
            #PersonalToolbar,
            #titlebar,
            toolbar#nav-bar,
            #urlbar,
            #urlbar-background {
              background: #1c1b22 !important;
            }

            /* squish everything into one bar
            :root {
              --navbarWidth: 25vw;
            }
            #navigator-toolbox {
              display: flex;
              flex-direction: row;
              flex-wrap: wrap;
            }
            #nav-bar {
              display: flex;
              flex-direction: row;
              order: 1;
              width: var(--navbarWidth);
            }
            #titlebar {
              order: 2;
              width: calc(95vw - var(--navbarWidth) - 1px);
            }
            #PersonalToolbar {
              order: 3;
              width: 100%;
            }
            toolbarbutton {
              padding: 0px !important;
            }
            #PanelUI-menu-button {
              order: 1;
            }
            #nav-bar-customization-target {
              order: 2;
            }
            */
          '';
        };
      };
    };
  };
}
