vim.opt.signcolumn = "no"
vim.opt.number = true
vim.opt.relativenumber = true
vim.api.nvim_create_autocmd({"TermOpen"}, {
  pattern = {"*"},
  command = "setlocal nonumber norelativenumber",
})

vim.opt.ignorecase = false
vim.opt.hlsearch = false
vim.opt.incsearch = true

vim.opt.scrolloff = 2
vim.opt.wrap = false

-- Popup menu height.
vim.opt.pumheight = 10

-- Mode indicator.
vim.opt.showmode = false
vim.opt.shortmess = "acFIoOsW"

local colors = {
  background = nil,
  white = "#FFFFFF",
  red = "#FF0000",
  yellow = "#FFFF00",
  black = "#000000",
  lightgrey = "#888888",
  blue = "#0000FF"
}
require("lualine").setup {
  options = {
    icons_enabled = true,
    theme = {
      normal = {
        a = { bg = colors.white, fg = colors.black },
        b = { bg = colors.lightgrey, fg = colors.white },
        c = { bg = colors.bg, fg = colors.white },
        y = { bg = colors.lightgrey, fg = colors.white },
        z = { bg = colors.white, fg = colors.black }
      },
      command = {
        a = { bg = colors.blue, fg = colors.black },
        b = { bg = colors.lightgrey, fg = colors.white },
        c = { bg = colors.bg, fg = colors.white },
        y = { bg = colors.lightgrey, fg = colors.white },
        z = { bg = colors.white, fg = colors.black }
      },
      insert = {
        a = { bg = colors.red, fg = colors.white },
        b = { bg = colors.lightgrey, fg = colors.white },
        c = { bg = colors.bg, fg = colors.white },
        y = { bg = colors.lightgrey, fg = colors.white },
        z = { bg = colors.white, fg = colors.black }
      },
      visual = {
        a = { bg = colors.yellow, fg = colors.black },
        b = { bg = colors.lightgrey, fg = colors.white },
        c = { bg = colors.bg, fg = colors.white },
        y = { bg = colors.lightgrey, fg = colors.white },
        z = { bg = colors.white, fg = colors.black }
      },
      replace = {
        a = { bg = colors.black, fg = colors.white },
        b = { bg = colors.lightgrey, fg = colors.white },
        c = { bg = colors.bg, fg = colors.white },
        y = { bg = colors.lightgrey, fg = colors.white },
        z = { bg = colors.white, fg = colors.black }
      }
    },
    component_separators = { left = "", right = ""},
    section_separators = { left = "", right = ""},
    ignore_focus = {},
    always_divide_middle = true,
    globalstatus = false,
    refresh = {
      statusline = 1000,
      tabline = 1000,
      winbar = 1000,
    }
  },
  sections = {
    lualine_a = {"mode"},
    lualine_b = {"branch", "diff"},
    lualine_c = {"diagnostics"},
    lualine_x = {},
    lualine_y = {"progress"},
    lualine_z = {"filetype", "filename"}
  },
  inactive_sections = {
    lualine_a = {},
    lualine_b = {"branch", "diff"},
    lualine_c = {},
    lualine_x = {},
    lualine_y = {},
    lualine_z = {"filetype", "filename"}
  },
  tabline = {},
  winbar = {},
  inactive_winbar = {},
  extensions = {}
}

vim.o.background = 'dark'
if vim.fn.exists('syntax_on') then
  vim.cmd('syntax reset')
end

vim.g.colors_name = 'translucent'
vim.api.nvim_set_hl(0, "FloatBorder", { fg = colors.white, bg = nil })
vim.api.nvim_set_hl(0, "PMenu", { ctermfg = "white", ctermbg = 242 })
vim.api.nvim_set_hl(0, "PMenuSbar", { ctermbg = 242 })
vim.api.nvim_set_hl(0, "PMenuSel", { ctermfg = 242, ctermbg = "white" })
vim.api.nvim_set_hl(0, "NormalFloat", { ctermfg = "white", ctermbg = nil })
vim.api.nvim_set_hl(0, "Search", { link = "Visual" })
