{ config, inputs, lib, pkgs, ... }:

let project-manager = inputs.project-manager.packages.x86_64-linux.default;
in with lib; {
  options.lwad.applications.projectManager = {
    enable = mkEnableOption "project-manager";
    projectsFile = mkOption {
      default = "\${HOME}/Documents/projects";
      description = "File to store project information in.";
      type = types.str;
    };
  };
  config = let cfg = config.lwad.applications.projectManager;
  in mkIf cfg.enable {
    environment = {
      systemPackages = [ project-manager ];
      sessionVariables.PROJECT_MANAGER_PATH = cfg.projectsFile;
    };
    programs = {
      bash = {
        interactiveShellInit = mkOrder 1 (let
          tmux = "${pkgs.tmux}/bin/tmux";
          test = "${pkgs.coreutils}/bin/test";
        in ''
          if ${test} -z "$TMUX"; then
            timeout=$((EPOCHSECONDS+2))
            if ! ${tmux} attach-session -t "base" 2>/dev/null && ${test} "$EPOCHSECONDS" -lt "$timeout"; then
              ${tmux} new-session -s "base"
            fi
            exit 0
          else
            if ${test} "$SHLVL" -eq 2 && ${test} -x ./.projectrc; then
              ./.projectrc
            fi
          fi
        '');
        shellAliases = {
          base =
            ''cd "$(tmux list-panes -F "#{pane_start_path}" | head -n 1)"'';
          p = "${project-manager}/bin/project-manager";
          pk = "tmux kill-session -t PROJECTS";
        };
      };
      tmux = {
        enable = true;
        escapeTime = 0;
        extraConfig =
          let projectManager = "${project-manager}/bin/project-manager";
          in ''
            set-option -g set-titles on
            set-option -g set-titles-string "#S"

            set -g status-position top
            set-option -g status-style bg=default
            set-option -g status-left ""
            set-option -g status-justify centre
            set-option -g window-status-format ""
            set-option -g window-status-current-format "#[bold]#{session_name}"

            bind -n m-f new-session -s "PROJECTS" -c "#{pane_start_path}" " ${projectManager}"
            bind -n m-b new-session -s "PROJECTS" " ${projectManager} -b"
            bind -n m-a new-session -s "PROJECTS" -c "#{pane_current_path}" " ${projectManager} -a"
            bind -n m-d new-session -s "PROJECTS" -c "#{pane_start_path}" " ${projectManager} -d"
            bind -n m-s new-session -s "STATUS" -c "#{pane_current_path}" " ${pkgs.btop}/bin/btop && ${pkgs.tmux}/bin/tmux switch-client -l"
            bind -n m-l detach-client
          '';
        keyMode = "vi";
      };
    };
  };
}
