{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.task = {
    categories = mkOption {
      default = [ ];
      description =
        "A list of task categories that can be viewed with a bash alias of the same name.";
      type = types.listOf types.str;
    };
    dataDir = mkOption {
      default =
        config.home-manager.users.${config.lwad.definitions.username}.home.homeDirectory
        + "/.tasks";
      description = "The directory to store task data in.";
      type = types.str;
    };
    enable = mkEnableOption "taskwarrior";
  };
  config = let cfg = config.lwad.applications.task;
  in mkIf cfg.enable {
    programs.bash = {
      interactiveShellInit = let
        cut = "${pkgs.coreutils}/bin/cut";
        git = "${pkgs.git}/bin/git";
        rg = "${pkgs.ripgrep}/bin/rg";
        task = "${pkgs.taskwarrior}/bin/task";
        tr = "${pkgs.coreutils}/bin/tr";
        wc = "${pkgs.coreutils}/bin/wc";
      in ''
        task () {
          if [[ "$1" == "sync" ]]
          then
            location="$(${task} diagnostics | ${rg} "Data:" | ${tr} -s " " | ${cut} -d " " -f 3)"
            pushd "$location" >/dev/null

            ${git} pull >/dev/null

            if ! [ "$(${git} status --porcelain | ${wc} -l)" -eq 0 ]
            then
              ${git} add . >/dev/null
              ${git} commit -m "chore: Sync on $(date)" >/dev/null
              ${git} push origin main >/dev/null
            fi
            popd >/dev/null
          else
            command ${task} $@
          fi
        }

        ${lib.concatStringsSep "\n" (map (category: ''
          ${category}() {
            if [[ $1 == "add" ]]; then
              shift
              ${task} add project:${category} $@
            else
              ${task} project:${category} $@
            fi
          }
        '') cfg.categories)}
      '';
    };
    home-manager.users.${config.lwad.definitions.username}.programs.taskwarrior =
      {
        config = {
          calendar.weekstart = "Monday";
          color.scheduled = "white on color4";
          news.version = lib.concatStringsSep "."
            ((lib.sublist 0 2 (lib.splitVersion pkgs.taskwarrior.version))
              ++ [ "0" ]);
          report."brief" = {
            columns = [
              "id"
              "project"
              "scheduled"
              "due"
              "depends"
              "description.count"
            ];
            filter = "status:pending project:uni";
            labels = [ "ID" "Project" "Scheduled" "Due" "Dep" "Description" ];
            sort = [ "priority-/" "project-" "description+" ];
          };
          terminal.editor = "nvim";
          uda.link = {
            label = "link";
            type = "string";
          };
        };
        dataLocation = cfg.dataDir;
        enable = true;
      };
    systemd.tmpfiles.rules =
      [ "d ${cfg.dataDir} - ${config.lwad.definitions.username} users" ];
  };
}
