{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.applications.terminalEmulator = {
    alacritty = {
      enable = mkEnableOption "alacritty";
      fontSize = mkOption {
        default = 10.0;
        description = "Font size to use in alacritty.";
        type = types.number;
      };
    };
    default = mkOption {
      default = null;
      description = "Default terminal emulator to use.";
      type = types.nullOr (types.enum [ "alacritty" "kitty" ]);
    };
    kitty = {
      enable = mkEnableOption "kitty";
      fontSize = mkOption {
        default = 10.0;
        description = "Font size to use in kitty.";
        type = types.number;
      };
    };
  };
  config.home-manager.users.${config.lwad.definitions.username}.programs = let
    cfg = config.lwad.applications.terminalEmulator;
    colors = config.lwad.definitions.theme.colors;
  in {
    alacritty = mkIf cfg.alacritty.enable {
      enable = true;
      package = pkgs.alacritty;
      settings = {
        colors = {
          primary = {
            background = colors.background;
            foreground = colors.text;
          };
          normal = {
            black = "#000000";
            red = "#D4160F";
            yellow = "#F8E960";
            white = colors.text;
          };
          bright = {
            black = "#000000";
            red = "#BE2A4A";
            yellow = "#F8E960";
            white = colors.text;
          };
        };
        font = {
          normal = {
            family = "FiraCode Nerd Font";
            style = "Regular";
          };
        };
      };
    };
    kitty = mkIf cfg.kitty.enable {
      enable = true;
      font.size = cfg.kitty.fontSize;
      settings = { # theming
        background = colors.background;
        color0 = "#000000";
        color1 = "#D4160F";
        color10 = "#B9CA4A";
        color11 = "#F8E960";
        color12 = "#7AA6DA";
        color13 = "#C397D8";
        color14 = "#70C0B1";
        color15 = colors.text;
        color2 = "#B5BD68";
        color3 = "#F8E960";
        color4 = "#81A2BE";
        color5 = "#B294BB";
        color6 = "#8ABEB7";
        color7 = colors.text;
        color8 = "#000000";
        color9 = "#BE2A4A";
        cursor = colors.text;
        foreground = colors.text;
        selection_background = colors.text;
        selection_foreground = colors.background;
      } // {
        confirm_os_window_close = 0;
        cursor_blink_interval = 0;
        cursor_shape = "block";
        disable_ligatures = "always";
        enable_audio_bell = "no";
        scrollback_lines = 0;
        shell_integration = "disabled";
        update_check_interval = 0;
      };
      font = {
        name = "FiraCode Nerd Font";
        package = pkgs.nerdfonts;
      };
    };
  };
}
