{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.definitions = {
    theme = {
      colors = {
        background = mkOption {
          default = "#333333";
          description = "Background color.";
          type = types.str;
        };
        highlight = mkOption {
          default = "#B4ACA4";
          description = "Highlight color.";
          type = types.str;
        };
        highlight_darker = mkOption {
          default = "#9B948D";
          description = "Dark variation of highlight color.";
          type = types.str;
        };
        menu_background = mkOption {
          default = "#555555";
          description = "Menu background color.";
          type = types.str;
        };
        near_black = mkOption {
          default = "#232122";
          description = "Near black color (can just be black).";
          type = types.str;
        };
        text = mkOption {
          default = "#FFFFFF";
          description = "Text color.";
          type = types.str;
        };
      };
      image = mkOption {
        default = ./topography.png;
        description = "Wallpaper image to use.";
        type = types.path;
      };
    };
    sudoas = {
      command = mkOption {
        default = null;
        description = "Either the sudo or doas command.";
        type = types.str;
      };
      variant = mkOption {
        default = "doas";
        description = "Which of sudo or doas to use.";
        type = types.enum [ "doas" "sudo" ];
      };
    };
    username = mkOption {
      default = "lukew";
      description = "Username of the primary user.";
      type = types.str;
    };
  };
  config = let cfg = config.lwad.definitions.sudoas.variant;
  in {
    lwad.definitions.sudoas.command = if (cfg == "doas") then
      "${config.security.wrapperDir}/${config.security.wrappers.doas.program}"
    else
      "${pkgs.sudo}/bin/sudo";
    programs.bash.interactiveShellInit = mkIf (cfg == "doas") ''
      complete -F _command doas
    '';
    security = {
      doas = mkIf (cfg == "doas") {
        enable = true;
        wheelNeedsPassword = false;
      };
      sudo.wheelNeedsPassword = mkIf (cfg == "sudo") false;
    };
  };
}
