{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.development.languages = {
    bash.enable = mkEnableOption "bash development packages";
    c.enable = mkEnableOption "c (and relatives) development packages";
    erlang.enable = mkEnableOption "erlang development packages";
    java.enable = mkEnableOption "java development packages";
    python.enable = mkEnableOption "python development packages";
    rust.enable = mkEnableOption "rust development packages";
    typescript.enable = mkEnableOption "typescript development packages";
  };
  config = let cfg = config.lwad.development.languages;
  in {
    environment.systemPackages = with pkgs;
      flatten [
        (optionals cfg.bash.enable [ shellcheck ])
        (optionals cfg.c.enable [ clang clangStdenv gcc gdb libclang ])
        (optionals cfg.erlang.enable [ erlang rebar3 ])
        (optionals cfg.java.enable [ pmd ])
        (optionals cfg.python.enable [ python310 ])
        (optionals cfg.rust.enable [ cargo rustc ])
        (optionals cfg.typescript.enable [ bun nodePackages.nodejs typescript ])
      ];
    programs.java = optionalAttrs cfg.java.enable {
      binfmt = true;
      enable = true;
    };
  };
}
