{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.groups.terminalCommands.enable =
    mkEnableOption "some useful terminal commands";
  config = mkIf config.lwad.groups.terminalCommands.enable {
    environment.systemPackages = with pkgs; [
      eza
      fd
      fzf
      jq
      rename
      ripgrep
      rsync
      watchexec
    ];
    programs.bash = {
      shellAliases = let eza = "${pkgs.eza}/bin/eza";
      in {
        eza = "${eza} --long --icons --git";
        recent = "${eza} --sort=modified --reverse | head -n 1";
        tree = "${eza} --long --icons --git --tree";
      };
      interactiveShellInit = ''
        cdn() {
          cd "$(dirname "$@")"
        }

        play() {
          if [[ "$1" == "" ]]; then
            target="."
          else
            target="$1"
          fi
          ${pkgs.mpv}/bin/mpv --video=no --display-tags-clr "$target"
        }

        ,() {
          if ${pkgs.coreutils}/bin/test -x ./.comma; then
            ./.comma $@
          fi
        }

        tell() {
          join_by() {
            local d=''${1-} f=''${2-}
            if shift 2; then
              ${pkgs.coreutils}/bin/printf %s "$f" "''${@/#/$d}"
            fi
          }

          session_name="$(${pkgs.tmux}/bin/tmux list-clients -t "$TMUX_PANE" -F "#{session_name}" | ${pkgs.coreutils}/bin/head -n 1)"

          if eval "$@"; then
            icon="data-information"
          else
            icon="data-error"
          fi

          ${pkgs.dunst}/bin/dunstify -a "longCommand" \
            -u normal \
            -i "$icon" \
            "$session_name" \
            "$(join_by " " $@)"
        }

        complete -F _command tell
      '';
    };
  };
}
