{ config, lib, ... }:

with lib; {
  options.lwad.hardening.nixDaemon.enable = mkOption {
    default = true;
    description = "Whether to enable nix-daemon hardening.";
    type = types.bool;
  };
  config.systemd.services.nix-daemon.serviceConfig =
    let cfg = config.lwad.hardening.nixDaemon.enable;
    in mkIf cfg {
      BindPaths = "/dev/kvm";
      CacheDirectory = "nix";
      CacheDirectoryMode = "0700";
      CapabilityBoundingSet =
        "CAP_CHOWN CAP_DAC_OVERRIDE CAP_DAC_READ_SEARCH CAP_FOWNER CAP_KILL CAP_SETGID CAP_SETUID CAP_SYS_ADMIN";
      DeviceAllow = "/dev/kvm";
      Environment = [ "XDG_CACHE_HOME=%C" ];
      NoNewPrivileges = true;
      PrivateDevices = true;
      ProtectClock = true;
      ProtectControlGroups = true;
      ProtectHome = "read-only";
      ProtectKernelModules = true;
      ProtectSystem = "strict";
      ReadWritePaths = "/nix /tmp";
      RestrictAddressFamilies = "AF_INET AF_INET6 AF_UNIX";
      RestrictNamespaces = "ipc mnt net pid user uts";
      # RestrictNetworkInterfaces = "~docker0 tailscale0";
      RestrictRealtime = true;
      RestrictSUIDSGID = true;
      SocketBindDeny = "any";
      SystemCallFilter =
        "@mount @pkey @system-service seccomp setdomainname sethostname";
    };
}
