{ config, lib, ... }:

with lib; {
  options.lwad.hardening.nixGc.enable = mkOption {
    default = true;
    description = "Whether to enable nix-gc hardening.";
    type = types.bool;
  };
  config = let cfg = config.lwad.hardening.nixGc.enable;
  in mkIf cfg {
    nix.settings.allowed-users = [ "nix-gc" ];
    systemd = {
      services.nix-gc = {
        confinement.enable = true;
        environment.HOME = "/home/nix-gc";
        serviceConfig = {
          BindPaths = "/nix/var/nix/profiles";
          BindReadOnlyPaths = "/nix/var/nix/daemon-socket";
          CapabilityBoundingSet = "";
          Group = "nix-gc";
          IPAddressDeny = "any";
          LockPersonality = true;
          MemoryDenyWriteExecute = true;
          NoNewPrivileges = true;
          PrivateNetwork = true;
          ProcSubset = "pid";
          ProtectClock = true;
          ProtectHome = "tmpfs";
          ProtectHostname = true;
          ProtectKernelLogs = true;
          ProtectProc = "invisible";
          RemoveIPC = true;
          RestrictAddressFamilies = "AF_UNIX";
          RestrictNamespaces = true;
          RestrictRealtime = true;
          RestrictSUIDSGID = true;
          SystemCallArchitectures = "native";
          SystemCallFilter = [ "@system-service" "~@resources @privileged" ];
          UMask = "0077";
          User = "nix-gc";
        };
      };
      tmpfiles.rules = [
        "A /nix/var/nix/profiles - - - - u:nix-gc:rwx,d:u:nix-gc:rwx,m::rwx,d:m::rwx"
      ];
    };
    users = {
      extraUsers.nix-gc = {
        group = "nix-gc";
        isSystemUser = true;
      };
      groups.nix-gc = { };
    };
  };
}
