{ config, lib, ... }:

with lib; {
  options.lwad.hardening.postgresql.enable = mkOption {
    default = true;
    description = "Whether to enable postgresql hardening.";
    type = types.bool;
  };
  config.systemd.services.postgresql =
    let cfg = config.lwad.hardening.postgresql.enable;
    in mkIf (cfg && config.services.postgresql.enable) {
      confinement = {
        enable = true;
        packages = [ config.i18n.glibcLocales ];
      };
      serviceConfig = let
        hosts = builtins.toFile "hosts" ''
          127.0.0.1 localhost
          ::1 localhost
        '';
      in {
        BindReadOnlyPaths = "${hosts}:/etc/hosts /etc/passwd";
        CapabilityBoundingSet = "";
        Environment =
          [ "XDG_DATA_DIRS=${config.services.postgresql.package}/share" ];
        IPAddressAllow = "localhost 100.76.94.126 100.113.1.60";
        IPAddressDeny = "any";
        LockPersonality = true;
        MemoryDenyWriteExecute = true;
        NoNewPrivileges = true;
        PrivateUsers = lib.mkForce false;
        ProcSubset = "pid";
        ProtectClock = true;
        ProtectHome = true;
        ProtectHostname = true;
        ProtectKernelLogs = true;
        ProtectKernelModules = true;
        ProtectProc = "invisible";
        RemoveIPC = true;
        RestrictAddressFamilies = "AF_UNIX AF_INET AF_INET6";
        RestrictNamespaces = true;
        RestrictNetworkInterfaces = "tailscale0 lo";
        RestrictRealtime = true;
        RestrictSUIDSGID = true;
        SystemCallArchitectures = "native";
        SystemCallFilter = [ "@system-service" "~@resources @privileged" ];
      };
    };
}
