{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.services.ci = {
    enable = mkEnableOption "act runner";
    hostPackages = mkOption {
      default = [ ];
      description = "List of packages to make available.";
      type = types.listOf types.package;
    };
    tokenFile = mkOption {
      default = "/etc/runner_token";
      description = "Path of the file to read the token from.";
      type = types.path;
    };
  };
  config.services.gitea-actions-runner.instances.runner =
    let cfg = config.lwad.services.ci;
    in mkIf cfg.enable {
      enable = true;
      hostPackages = cfg.hostPackages ++ (with pkgs; [ git openssh ]);
      labels = [ "native:host" ];
      name = config.networking.hostName;
      settings = {
        cache.enabled = true;
        host.workdir_parent = "/var/lib/gitea-runner/action-cache-dir";
      };
      tokenFile = cfg.tokenFile;
      url = "https://git.disroot.org";
    };
}
