{ config, lib, pkgs, ... }:

with lib; {
  options.lwad.services.dunst.enable = mkEnableOption "dunst";
  config.home-manager.users.${config.lwad.definitions.username}.services.dunst =
    let
      cfg = config.lwad.services.dunst.enable;
      colors = config.lwad.definitions.theme.colors;
    in mkIf cfg {
      enable = true;
      package = pkgs.dunst;
      settings = {
        global = {
          monitor = 0;
          follow = "none";
          width = 300;
          height = 300;
          origin = "top-right";
          offset = "10x10";
          scale = 0;
          notification_limit = 10;
          progress_bar = true;
          highlight = colors.highlight;
          progress_bar_height = 10;
          progress_bar_frame_width = 1;
          progress_bar_min_width = 150;
          progress_bar_max_width = 300;
          indicate_hidden = "yes";
          transparency = 0;
          separator_height = 2;
          padding = 8;
          horizontal_padding = 8;
          text_icon_padding = 0;
          frame_width = 3;
          frame_color = colors.text;
          separator_color = "foreground";
          sort = "yes";
          font = "FiraCode Nerd Font 11";
          line_height = 0;
          markup = "full";
          format = "<b>%s</b>\\n%b";
          alignment = "left";
          vertical_alignment = "center";
          show_age_threshold = 60;
          ellipsize = "middle";
          ignore_newline = "no";
          stack_duplicates = "true";
          hide_duplicate_count = "false";
          show_indicators = "yes";
          icon_position = "left";
          min_icon_size = 0;
          max_icon_size = 32;
          icon_path =
            let base_path = "${pkgs.breeze-icons}/share/icons/breeze-dark";
            in "${base_path}/status/16/:${base_path}/devices/16";
          sticky_history = "yes";
          history_length = 20;
          dmenu = "${pkgs.rofi}/bin/rofi -dmenu -p dunst:";
          browser = "${pkgs.xdg-utils}/bin/xdg-open";
          always_run_script = "true";
          title = "Dunst";
          class = "Dunst";
          corner_radius = 0;
          ignore_dbusclose = "false";
          force_xwayland = "false";
          force_xinerama = "false";
          mouse_left_click = "close_current";
          mouse_middle_click = "close_all";
          mouse_right_click = "do_action";
        };
        experimental.per_monitor_dpi = "false";
        urgency_low = {
          background = colors.near_black;
          foreground = colors.text;
          timeout = 10;
        };
        urgency_normal = {
          background = colors.menu_background;
          foreground = colors.text;
          timeout = 10;
        };
        urgency_critical = {
          background = colors.highlight_darker;
          foreground = colors.text;
          frame_color = colors.highlight;
          timeout = 0;
        };
        play_sound = {
          summary = "*";
          script = builtins.toString (pkgs.writeScript "sound.sh" (let
            mkSound =
              "${pkgs.mpv}/bin/mpv ${pkgs.sound-theme-freedesktop}/share/sounds/freedesktop/stereo/bell.oga --really-quiet";
          in ''
            case "$DUNST_APP_NAME" in
              changeVolume)
                ;;
              activity-timer)
                while ${pkgs.coreutils}/bin/test "$(${pkgs.dunst}/bin/dunstctl count displayed)" -ne 0; do
                  ${mkSound}
                done
                ;;
              *)
                  ${mkSound}
                ;;
            esac
          ''));
        };
      };
    };
}
