{ config, lib, pkgs, ... }:

with lib;
let
  snapshotOptions = {
    options = {
      snapshotDir = mkOption {
        description = "The directory to save snapshots to.";
        type = types.str;
      };
      subvolume = mkOption {
        description = "The subvolume to snapshot.";
        type = types.str;
      };
      sync = mkOption {
        default.enable = false;
        description = "Server synchronisation options.";
        type = types.submodule syncOptions;
      };
      volume = mkOption {
        description = "The volume containing the subvolume.";
        type = types.str;
      };
    };
  };
  syncOptions = {
    options = {
      enable = mkEnableOption "server synchronisation";
      serverAddress = mkOption {
        description = "Address of btrbk server to sync to.";
        type = types.str;
      };
      serverBackupDir = mkOption {
        description = "Directory on btrbk server to sync to.";
        type = types.str;
      };
    };
  };
in {
  options.lwad.utilities.btrbk.snapshots = mkOption {
    default = [ ];
    description = "Snapshots to take with btrbk.";
    type = types.listOf (types.submodule snapshotOptions);
  };
  config = let cfg = config.lwad.utilities.btrbk.snapshots;
  in mkIf (length cfg != 0) {
    services.btrbk = {
      extraPackages = with pkgs; [ zstd ];
      instances.${config.networking.hostName} = {
        onCalendar = "daily";
        settings = {
          send_compressed_data = "yes";
          snapshot_preserve = "14d 3m";
          snapshot_preserve_min = "2d";
          ssh_user = "btrbk";
          stream_buffer = "25%";
          stream_buffer_remote = "25%";
          stream_compress = "zstd";
          target_preserve = "3w 3m";
          target_preserve_min = "no";
          transaction_syslog = "user";
          volume = mkMerge (map (entity: {
            ${entity.volume}.subvolume.${entity.subvolume} = {
              snapshot_dir = entity.snapshotDir;
              target."ssh://${entity.sync.serverAddress}${entity.sync.serverBackupDir}" =
                mkIf entity.sync.enable { };
            };
          }) cfg);
        };
      };
    };
    systemd.services."btrbk-${config.networking.hostName}".restartIfChanged =
      false;
  };
}
