{ config, pkgs, ... }:

{
  hardware.bluetooth.enable = false;
  home-manager.users.${config.lwad.definitions.username} = {
    home.stateVersion = "22.11";
    services.cbatticon.enable = true;
  };
  imports = [ ./hardware-configuration.nix ../lwad ];
  lwad = {
    applications.i3 = {
      audioControls = {
        commands = let
          mute_audio = pkgs.writeScript "mute_audio.sh" ''
            IFS=$'\n'
            sinks=$(${pactl} list short sinks | cut -f 1)
            for sink in $sinks; do
              ${pactl} set-sink-volume $sink 0
            done
          '';
          pactl = "${pkgs.pulseaudio}/bin/pactl";
        in {
          headphones = ''
            ${mute_audio} && ${pactl} set-default-sink $(${pactl} list sinks | rg "Name: " | cut -d " " -f 2 | rg "Focusrite_Scarlett_2i2") && ${pkgs.pamixer}/bin/pamixer --set-volume 0'';
          speakers = ''
            ${mute_audio} && ${pactl} set-default-sink $(${pactl} list sinks | rg "Name: " | cut -d " " -f 2 | rg "analog-stereo") && ${pkgs.pamixer}/bin/pamixer --set-volume 0'';
        };
        enable = true;
      };
      backlight.enable = true;
      displayProperties = {
        internal = {
          width = 1366;
          height = 768;
          output = "LVDS-1";
        };
        ultrawide = {
          width = 2560;
          height = 1080;
          output = "HDMI-1";
        };
        panasonic = {
          width = 1280;
          height = 720;
          output = "HDMI-1";
          extraArgs = ''
            --set underscan on --set "underscan hborder" 34 --set "underscan vborder" 19'';
        };
      };
      outputWorkspaceMappings = {
        "LVDS-1 HDMI-1" = [ 1 2 3 4 5 ];
        "HDMI-1 LVDS-1" = [ 6 7 8 9 10 ];
      };
    };
    filesystems.simple.enable = true;
    utilities.btrbk.server.enable = true;
  };
  networking.hostName = "shinobu";
  nixpkgs.overlays = [
    (self: super: {
      bun = super.bun.overrideAttrs (old: rec {
        src = pkgs.fetchurl {
          hash = "sha256-XIOkHAc5HwbD4S/yT+GgZc453Py4WVueabv+eTYcHqk=";
          url =
            "https://github.com/oven-sh/bun/releases/download/bun-v${version}/bun-linux-x64-baseline.zip";
        };
        version = "1.0.23";
      });
    })
  ];
  programs = {
    light.enable = true;
    tmux = {
      extraConfigBeforePlugins = ''
        set-option -g @batt_icon_status_charged " "
        set-option -g @batt_icon_status_charging " ▲"
        set-option -g @batt_icon_status_discharging " ▼"
        set-option -g @batt_icon_status_attached " "
        set-option -g @batt_icon_status_unknown " ▲"
        set-option -g status-right "#{battery_percentage}#{battery_icon_status}"
      '';
      plugins = with pkgs.tmuxPlugins; [ battery ];
    };
  };
  services = {
    logind = {
      extraConfig = ''
        HandlePowerKey=ignore
      '';
      lidSwitch = "ignore";
    };
    xserver = {
      libinput.touchpad = {
        disableWhileTyping = true;
        naturalScrolling = true;
      };
      synaptics = {
        maxSpeed = "1";
        minSpeed = "1";
        palmDetect = true;
        twoFingerScroll = true;
      };
    };
  };
  system.stateVersion = "22.11";
}
